@@ -171,8 +171,9 @@ struct xatmibuf {
   xatmibuf(const char *type, long len) : pp(&p), len(len), p(nullptr) {
     reinit(type, len);
   }
-  void reinit(const char *type, long len) {
+  void reinit(const char *type, long len_) {
     if (*pp == nullptr) {
+      len = len_;
       *pp = tpalloc(const_cast<char *>(type), nullptr, len);
       if (*pp == nullptr) {
         throw std::bad_alloc();
@@ -212,7 +213,8 @@ struct xatmibuf {
       int rc = f(*fbfr());
       if (rc == -1) {
         if (Ferror32 == FNOSPACE) {
-          *pp = tprealloc(*pp, Fsizeof32(*fbfr()) * 2);
+          len *= 2;
+          *pp = tprealloc(*pp, len);
         } else {
           throw fml32_exception(Ferror32);
         }
@@ -366,8 +368,8 @@ static void from_py1(xatmibuf &buf, FLDID32 fieldid, FLDOCC32 oc,
     std::string val(PyString_AsString(obj.ptr()), PyString_Size(obj.ptr()));
 #endif
     buf.mutate([&](FBFR32 *fbfr) {
-      return CFchg32(fbfr, fieldid, oc, const_cast<char *>(val.data()), 0,
-                     FLD_STRING);
+      return CFchg32(fbfr, fieldid, oc, const_cast<char *>(val.data()),
+                     val.size(), FLD_CARRAY);
     });
   } else if (py::isinstance<py::int_>(obj)) {
     long val = obj.cast<py::int_>();
@@ -1144,6 +1146,7 @@ PYBIND11_MODULE(tuxedo, m) {
   m.attr("FLD_DOUBLE") = py::int_(FLD_DOUBLE);
   m.attr("FLD_STRING") = py::int_(FLD_STRING);
   m.attr("FLD_CARRAY") = py::int_(FLD_CARRAY);
+  m.attr("FLD_FML32") = py::int_(FLD_FML32);
   m.attr("BADFLDID") = py::int_(BADFLDID);
 
   m.attr("TPEX_STRING") = py::int_(TPEX_STRING);
